#!/bin/sh

# Network configuration. To set an interface static, IP and netmask must be set

# WAN Interface
ETH0_IP=''
ETH0_NETMASK=''
ETH0_GATEWAY=''

# LAN Interface
ETH1_IP=''
ETH1_NETMASK=''
ETH1_GATEWAY=''

# LAN2 Interface
ETH2_IP=''
ETH2_NETMASK=''
ETH2_GATEWAY=''

# Other
DEVICE_HOSTNAME=''
DNS_SERVERS=''
SEARCH_DOMAINS=''

# Cloud Configuration. To enable it username and password must be set either
# here or inside an exported config file

CLOUD_SERVER=''
CLOUD_USERNAME=''
CLOUD_PASSWORD=''

# Mobile/PPP Setup and authentication
MOBILE_APN=''
MOBILE_PIN=''
MOBILE_AUTHTYPE=''      # 1: PAP
MOBILE_USERNAME=''
MOBILE_PASSWORD=''

export PATH=$PATH:/bin/:/sbin/:/usr/bin/:/usr/sbin/:
cd $(dirname $0)

API_URL=http://localhost:3000/api/v1
LOG_FILE=lastRun.log

exec &>$LOG_FILE

callAPI() {

	module="$1"
	postData="$2"

	echo Calling API $API_URL/$module
	echo POSTDATA: $postData 
	echo

	bash -c "wget --header='Content-type:application/json;charset=utf-8' -O /dev/null --post-data '$postData' $API_URL/$module"
	ret=$?
	
	echo
	return $ret
}

msg() {

	if [ $# -lt 1 ]; then
		psplashBuffer=""
		echo
		return
	fi

	while [ $# -gt 0 ]; do
		echo -e $1
		psplashBuffer="${psplashBuffer}\n$1"
		shift
	done

	[ ! -e "/tmp/psplash_fifo" ] && return
	
	psplash-write "MSG $( echo -e $psplashBuffer )


"
	sleep 1
}

die(){

	cp /var/log/messages messages.lastFail

	msg	"\n$@"
	sync
	msg	"Log saved to \"$LOG_FILE\""
	
	while [ "1" ]; do
		dbus-send --system --print-reply --dest=com.exor.EPAD "/Buzzer" com.exor.EPAD.Buzzer.beep int32:440 int32:100; sleep 2;
	done
	
	exit 1
}


# The USB stick can be plugged before power on. 
# Wait for JMUconfig to start
while [ ! -e "/var/run/jmuconfig.pid" ]; do
	sleep 3;
done

# Test server is ready
while ( ! wget $API_URL/system -O /dev/null &>/dev/null ); do
	sleep 5
done

dbus-send --system --print-reply --dest=com.exor.EPAD "/Buzzer" com.exor.EPAD.Buzzer.beep int32:440 int32:100; sleep 1;

# Start psplash. On blind devices it will just fail to start
chvt 1
psplash --notouch &

echo

#
# 1. Cloud Configuration 
#

msg "- SETTING Cloud SERVICE:"

# Read Cloud parameters from config file if found
configFile="$( ls | grep -m 1 "^config_" )"
if [ -e "$configFile" ]; then

	echo "Found Cloud configuration file: $configFile"
	echo

	[ -z "$CLOUD_USERNAME" ] &&  CLOUD_USERNAME="` cat $configFile | grep -m 1 "^NAME:" | cut -d' ' -f2 | tr -d '\r' `"
	[ -z "$CLOUD_PASSWORD" ] && CLOUD_PASSWORD="` cat $configFile | grep -m 1 "^PASSWORD:" | cut -d' ' -f2 | tr -d '\r' `"
fi

# We need at least a username and a password
if [ -n "$CLOUD_USERNAME" -a -n "$CLOUD_PASSWORD" ]; then

	# Use default server if not specified
	[ -z "$CLOUD_SERVER" ] && CLOUD_SERVER="switchboard-exor1-1.endian.com"

	msg	"  server: $CLOUD_SERVER" \
		"  username: $CLOUD_USERNAME"

	callAPI services "{\"status\": [{\"id\": \"router\", \"name\": \"Router Service\", \"running\": true}]}"
	[ $? -ne 0 ] && die "ERROR: Failed to start Router service"

	# On older BSPs we need to start the Cloud service before we can configure it
	callAPI services "{\"status\": [{\"id\": \"encloud\", \"name\": \"Cloud Service\", \"running\": true}]}"

	callAPI services/cloud "{\"login\":{\
		\"server\":\"${CLOUD_SERVER}\",\
		\"type\":\"cloud\",\
		\"auth_type\":\"user-pass\",\
		\"user\":\"${CLOUD_USERNAME}\",\
		\"pass\":\"${CLOUD_PASSWORD}\"}}"

	[ $? -ne 0 ] && die "ERROR: Failed to set Cloud settings"

	callAPI services "{\"status\": [{\"id\": \"encloud\", \"name\": \"Cloud Service\", \"running\": true}]}"
	[ $? -ne 0 ] && die "ERROR: Failed to start Cloud service"

else
	msg '  Missing parameters, leaving service unconfigured.'
fi


#
# 2. Network Configuration 
#

msg '\n- SETTING NETWORK INTERFACES:'

for iface in $( ls /sys/class/net | grep eth ); do

	if [ "$iface" == "eth0" ]; then 
		label="WAN"
	else
		label="LAN${iface:3}"
	fi
	
	ip=$( eval echo \${${iface^^}_IP} )
	netmask=$( eval echo \${${iface^^}_NETMASK} )
	gateway=$( eval echo \${${iface^^}_GATEWAY} )

	if [ -n "$ip" -a -n "$netmask" ]; then
		netMessage="${netMessage}  ${iface} IP:$ip NM:$netmask\n"

		config="{\"name\":\"${iface}\",\"dhcp\":false,\"ip_address\":\"${ip}\",\"netmask\":\"${netmask}\",\"label\":\"${label}\""
		[ -n "$gateway" ] && config="${config},\"gateway_ip\":\"${gateway}\""
		config="${config}}"
	else
        netMessage="${netMessage}  ${iface} DHCP\n"

        config="{\"name\":\"${iface}\",\"dhcp\":true, \"label\":\"${label}\"}"
	fi
	
	[ -n "$networkData" ] && networkData="${networkData},"
	networkData="${networkData}${config}"
	
done

if [ -n "$DNS_SERVERS" -o -n "$SEARCH_DOMAINS" ]; then

	for dns in $DNS_SERVERS; do
		[ -n "$dnsServers" ] && dnsServers="${dnsServers},"
		dnsServers="${dnsServers}{\"ip\": \"${dns}\"}"
	done
	
	for domain in $SEARCH_DOMAINS; do
		[ -n "$domains" ] && domains="${domains},"
		domains="${domains}{\"name\": \"${domain}\"}"
	done
	
	if [ -n "$DNS_SERVERS" ]; then 
		dnsData="\"servers\": [${dnsServers}]"
		[ -n "$SEARCH_DOMAINS" ] && dnsData="${dnsData},\"search\": [${domains}]"
	else
		dnsData="\"search\": [${domains}]"
	fi
fi

networkPost="{ \"bridge\": { \"enabled\": false }, \"interfaces\":[${networkData}]"

if [ -n "$DEVICE_HOSTNAME" ]; then
	networkPost="${networkPost}, \"hostname\": \"${DEVICE_HOSTNAME}\""
	netMessage="${netMessage}  HOSTNAME: ${DEVICE_HOSTNAME}\n"
fi

[ -n "$dnsData" ] && networkPost="${networkPost}, \"dns\": {${dnsData}}"

networkPost="${networkPost}}"

[ -n "$DNS_SERVERS" ] && netMessage="${netMessage}  DNS: ${DNS_SERVERS}\n"
[ -n "$SEARCH_DOMAINS" ] && netMessage="${netMessage}  SEARCH DOMAINS: ${SEARCH_DOMAINS}\n"

msg "$netMessage"

msg '\n- SETTING MOBILE PARAMETERS'

[ -z "${MOBILE_APN}" ] || sys_params -w "services/mobile/autostart" "true"
[ -z "${MOBILE_APN}" ] || sys_params -w "services/mobile/apn" "${MOBILE_APN}"
[ -z "${MOBILE_PIN}" ] || sys_params -w "services/mobile/pin" "${MOBILE_PIN}"
[ -z "${MOBILE_AUTHTYPE}" ] || sys_params -w "services/mobile/auth/type" "${MOBILE_AUTHTYPE}"
case "${MOBILE_AUTHTYPE}" in
    1) 
        sys_params -w "services/mobile/auth/user" "${MOBILE_USERNAME}"
        sys_params -w "services/mobile/auth/pass" "${MOBILE_PASSWORD}"
        ;;
    *)
        ;;
esac

callAPI network "$networkPost"
[ $? -ne 0 ] && die "ERROR: Failed to configure network interfaces"

sync

msg	"\nOPERATION COMPLETED."

dbus-send --system --print-reply --dest=com.exor.EPAD "/Buzzer" com.exor.EPAD.Buzzer.beep int32:440 int32:100; sleep 1;
dbus-send --system --print-reply --dest=com.exor.EPAD "/Buzzer" com.exor.EPAD.Buzzer.beep int32:440 int32:100; sleep 1;
dbus-send --system --print-reply --dest=com.exor.EPAD "/Buzzer" com.exor.EPAD.Buzzer.beep int32:440 int32:100; sleep 1;

sleep 3

killall -INT psplash
